/*
This program has been developed by students from the bachelor Computer Science at Utrecht University within the Software Project course.
© Copyright Utrecht University (Department of Information and Computing Sciences)
*/

package produce

import (
	"context"
	"fmt"
	"query-service/pkg/logger"

	"git.science.uu.nl/GraphPolaris/keyvaluestore"
)

/*
PublishMessage will publish the message to the queue retrieved from the key value store, with the given sessionID
	data: *[]byte, the message to be published
	sessionID: *string, the ID of the session
*/
func (s *Service) PublishMessage(data *[]byte, sessionID *string) {
	// Use the sessionID to query the key value store to get the queue we need to send this message to
	clientQueueID, err := s.keyValueStore.Get(context.Background(), *sessionID, keyvaluestore.String)
	if err != nil || clientQueueID == nil {
		return
	}

	logger.Log(fmt.Sprintf("Found client queue %s for session %s", clientQueueID, *sessionID))

	// If this client has now disconnected
	if clientQueueID == "" {
		// TODO: Decide whether to throw away the message or perhaps cache it, for now throw it away
		logger.Log("No client updater queue ID")
		return
	}

	headers := make(map[string]interface{})
	headers["sessionID"] = *sessionID
	headers["type"] = "queryResult"
	s.producerDriver.PublishMessage(data, clientQueueID.(string), &headers)
}
